module sourcematch_mod
    use param
    use cal
    use dal
 
  use oal
  use caloaldefs
  

  
  use errorHandling

    use sasdal
  
    
    use utility_mod  
contains


!***************************************************
! Function SourceMatch
! Tries to match a catalogue star to a given source
!
!***************************************************
logical function SourceMatch(raXMM, decXMM, magXMM, &
                           size, raCat, decCat, magCat, tolerance, &
                           maxMagDifference, minRadius, index, likelihood)

 
    implicit none 
    real(kind=double), intent(in) :: raXMM, decXMM
    real(kind=single), intent(in) ::  magXMM      ! XMM ra, dec & magnitude
    
    integer(kind=int32), intent(in) :: size

    real(kind=double), intent(in), dimension(1 : size) :: raCat, decCat
    real(kind=single), intent(in), dimension(1 : size) :: magCat
!    type(CatalogueData), intent(in), dimension(1 : size) :: catData
    real(kind=single), intent(in) :: tolerance
  
   
    real(kind=single), intent(in) :: maxMagDifference 
    integer, intent(out) :: index     
    real(kind=double), intent(out) :: likelihood
    real(kind=single), intent(out) :: minRadius
    real(kind=double) :: distance
    real(kind=double) :: delta
    real(kind=double) :: sigmaSquared
    real(kind = double), parameter :: PI = 3.141592653589793_double  
 
    real(kind = double), parameter         :: degreesToRadians = PI / 180.0_double

    integer i, kl, ku
    integer  :: jStart
 
    jStart = 1   
    delta = (tolerance / 3600.0_double) * degreesToRadians 
    sigmaSquared = tolerance * tolerance
    kl = Locate(size, raCat, raXMM - delta, jStart)
    ku = Locate(size, raCat, raXMM + delta, jStart)
    
                    
    minRadius = 10000.0
    SourceMatch = .false.
    likelihood = 0.0_double

    do i = kl, ku
       
        if (abs(decCat(i) - decXMM) .gt. delta) cycle
        

        !*******************************************
        ! Check the magnitudes
        !******************************************* 
        if(abs(magCat(i) - magXMM) .le. maxMagDifference) then
            
            distance = ArcLength(raXMM, decXMM, raCat(i), decCat(i))
            if (distance .le. tolerance) then  
                likelihood = likelihood + exp(-0.5_double * distance * distance / sigmaSquared) 
 
           

                if(distance .lt. minRadius) then
                    minRadius = distance
                    SourceMatch = .true.
                    index = i
                end if

                
            end if
        end if
    end do
   
end function SourceMatch   

!********************************************
! Function Locate
! Given an array table(1 .. n), and
! given a value x, returns the index such that 
! x is between table(j) and table(j+1)
! The search starts at the position jStart
!********************************************
integer function Locate(size, table, searchValue, jStart)
    implicit none
    integer, intent(in) :: size
    real(kind=double), intent(in), dimension(size) :: table
    real(kind=double), intent(in) :: searchValue
    integer, intent(inout) :: jStart
    integer :: k, klo, khi

    !********************************************
    ! Check to see if x is less than the minimum
    ! value in the table
    !********************************************
    if (searchValue .lt. table(1)) then
        Locate = 1
        return
    end if
  
    !***************************************
    ! Check to see if x is greater than the 
    ! maximum value in the table
    !***************************************
    if (searchValue .gt. table(size)) then
        Locate = size
        return
    end if

    klo = jStart
    khi = size

    do while((khi - klo) .gt. 1)
        k = (khi + klo) / 2
        if (table(k) .gt. searchValue) then
            khi = k
        else
            klo = k
        end if
    end do

    !*********************************
    ! klo and khi now bracket the 
    ! searchvalue
    !*********************************
    Locate = klo
    jStart = klo
end function Locate 

end module sourcematch_mod








