!*****************************************************************
!
!
!****************************************************************
module statistics_mod
    use param
    use cal
    use dal
 
  use oal
  use caloaldefs
  

  
  use errorHandling
    use sasdal
  
 
contains
!*****************************************************************
!
!
!****************************************************************
subroutine Statistics(nData, xData, weight, mean, meanStandardError, &
                     standardDeviation, s3, s4, xmin, xmax)
 

    integer, intent(in) :: ndata 
    real(kind=double), intent(in), dimension(ndata) ::  xData, weight
    real(kind=double), intent(out) :: mean
    real(kind=double), intent(out) :: meanStandardError
    real(kind=double), intent(out) :: standardDeviation
    real(kind=double), intent(out) :: xmin, xmax
    real(kind=double), intent(out) ::  s3, s4
    real(kind=double) :: wtsum
  
    integer i, ifail, iwt

    ifail = 1
    iwt = 1  ! No weighting
!    call g01aaf(nData, xData, iwt, weight, mean, standardDeviation, s3, s4, &
!                xmin, xmax, wtsum, ifail)
   
    if (ifail .ne. 0) write(*, *) "Error calling g01aaf!- ifail=", ifail
    write(*, *) "No of data=", ndata
    write(*, *) "Mean=", mean
    meanStandardError = standardDeviation / dsqrt(wtsum)
    write(*, *) "Standard error of mean=", meanStandardError
    write(*, *) "Standard deviation=", standardDeviation
    write(*, *) "Coefficient of skewness=", s3
    write(*, *) "Coefficient of kurtosis=", s4
    
end subroutine Statistics
!************************************
!
! Calculates a 95% confidence interval for mean
!************************************
subroutine MeanConfidenceInterval(ndata, mean, standardDeviation,&
                                  meanLower, meanUpper)
    integer, intent(in) :: ndata
    real(kind=double), intent(in) :: mean, standardDeviation
    real(kind=double), intent(out) :: meanLower, meanUpper
    real(kind=double) :: interval

    interval = 1.96d0 * standardDeviation * standardDeviation / dsqrt(dble(ndata))
    meanLower = mean - interval
    meanUpper = mean + interval
end subroutine MeanConfidenceInterval
!****************************************
! Subroutine MeanTest - Test whether a mean is significantly 
! different from a given value
!*****************************************
subroutine Meantest(ndata, mean, standardDeviation, value, zStatistic)
    integer, intent(in) :: ndata
    real(kind=double), intent(in) :: mean, standardDeviation, value
    real(kind=double), intent(out) :: zStatistic

    zStatistic = (mean - value) / (standardDeviation / dsqrt(dble(ndata)))
end subroutine MeanTest 
!*****************************************************
! Subroutine DrawNormalDistribution
!*****************************************************
subroutine DrawNormalDistribution(noData, binWidth, mean, standardDeviation)
    integer, intent(in) :: noData
    real(kind=double), intent(in) :: mean, standardDeviation
    real(kind=double) :: variance
    real(kind=single), allocatable, dimension(:) :: xData, yData
    integer i
    real(kind=double) xValue, yValue, dx
    real xmin, xmax, ymin, ymax
    integer, parameter :: size = 100
    real(kind=double), intent(in) ::  binWidth
    real(kind=double) ::  constant
    real(kind = double), parameter :: PI = 3.141592653589793_double
    allocate(xData(1 : size))
    allocate(yData(1 : size))

    variance = standardDeviation * standardDeviation
    constant = dble(noData) * binWidth / (standardDeviation * sqrt(2.0_double * PI))
!write(*, *) "DrawNormalDistribution-binWidth=", binWidth, " constant=", constant


    call pgqwin(xmin, xmax, ymin, ymax)
    dx = (xmax - xmin) / real(size - 1)
    do i = 1, size

        xValue = xmin + dx * real(i  - 1)
        yValue = constant * Gaussian(xValue, mean, variance)

        xData(i) = xValue         
        yData(i) = yValue
    end do
    call pgline(size, xData, yData)
    deallocate(xData)
    deallocate(yData)
end subroutine DrawNormalDistribution
!*******************************************
! Function Gaussian
!*******************************************
real(kind=double) function Gaussian(x, mean, variance)
    implicit none
    real(kind=double), intent(in) :: x, mean, variance
    
    Gaussian = exp(-((x - mean) ** 2) / &
               (2.0_double * variance))
end function Gaussian
end module statistics_mod


























